package cs108;

import javafx.application.Application;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.geometry.VPos;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.stage.Stage;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public final class Main extends Application {
    private final static int PIECE_SIDE = 50;
    private final static int INTER_PIECE_SPACE = 3;

    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage primaryStage) {
        ObjectProperty<Puzzle> puzzleP = new SimpleObjectProperty<>();

        Map<Piece, Node> pieceNode = new HashMap<>();
        for (Piece piece : Piece.values()) {
            if (piece == Piece.HOLE) continue;

            Node node = nodeForPiece(piece);
            pieceNode.put(piece, node);

            node.setOnMouseClicked(e -> {
                Puzzle puzzle = puzzleP.get();
                int pIndex = puzzle.pieces().indexOf(piece);
                if (puzzle.canSwap(pIndex))
                    puzzleP.set(puzzle.withPieceSwapped(pIndex));
            });
        }

        Text motivator = new Text();
        motivator.textProperty().bind(
                puzzleP.map(p -> p.isSolved() ? "Bravo !" : "A résoudre..."));

        Pane gamePane = new Pane();
        gamePane.getChildren().setAll(pieceNode.values());
        double prefSide = PIECE_SIDE * Puzzle.SIDE_LENGTH + INTER_PIECE_SPACE * (Puzzle.SIDE_LENGTH + 1);
        gamePane.setPrefSize(prefSide, prefSide);

        Button shuffleButton = new Button("Mélanger !");
        shuffleButton.setOnAction(_ -> puzzleP.set(puzzleP.get().shuffled()));

        VBox mainPane = new VBox(motivator, gamePane, shuffleButton);
        mainPane.setSpacing(3);
        mainPane.setPadding(new Insets(5));
        mainPane.setAlignment(Pos.CENTER);

        puzzleP.addListener((_, _, n) -> layoutPuzzle(pieceNode, n));
        puzzleP.set(Puzzle.SOLVED);

        primaryStage.setScene(new Scene(mainPane));
        primaryStage.setTitle("Taquin");
        primaryStage.show();
    }

    private Node nodeForPiece(Piece piece) {
        assert piece != Piece.HOLE;

        Rectangle r = new Rectangle(PIECE_SIDE, PIECE_SIDE);
        r.setFill(Color.SALMON);
        Text t = new Text(5, 5, piece.toString());
        t.setTextOrigin(VPos.TOP);
        return new Group(r, t);
    }

    private void layoutPuzzle(Map<Piece, Node> pieceNode, Puzzle puzzle) {
        Iterator<Piece> pieceIt = puzzle.pieces().iterator();
        for (int r = 0; r < Puzzle.SIDE_LENGTH; r += 1) {
            int y = INTER_PIECE_SPACE + r * (PIECE_SIDE + INTER_PIECE_SPACE);
            for (int c = 0; c < Puzzle.SIDE_LENGTH; c += 1) {
                int x = INTER_PIECE_SPACE + c * (PIECE_SIDE + INTER_PIECE_SPACE);

                Node n = pieceNode.get(pieceIt.next());
                if (n == null) continue;

                n.relocate(x, y);
            }
        }
    }
}
